@@ -1153,6 +1153,30 @@ void bind_alert()
         .add_property("metadata", make_getter(&torrent_conflict_alert::metadata, by_value()))
         ;
 
+    class_<peer_info_alert, bases<torrent_alert>, noncopyable>(
+        "peer_info_alert", no_init)
+        .add_property("peer_info", make_getter(&peer_info_alert::peer_info, by_value()))
+        ;
+
+    class_<file_progress_alert, bases<torrent_alert>, noncopyable>(
+        "file_progress_alert", no_init)
+        .add_property("files", make_getter(&file_progress_alert::files, by_value()))
+        ;
+
+    class_<piece_info_alert, bases<torrent_alert>, noncopyable>(
+        "piece_info_alert", no_init)
+        .add_property("piece_info", make_getter(&piece_info_alert::piece_info, by_value()))
+        ;
+
+    class_<piece_availability_alert, bases<torrent_alert>, noncopyable>(
+        "piece_availability_alert", no_init)
+        .add_property("piece_availability", make_getter(&piece_availability_alert::piece_availability, by_value()))
+        ;
+
+    class_<tracker_list_alert, bases<torrent_alert>, noncopyable>(
+        "tracker_list_alert", no_init)
+        .add_property("trackers", make_getter(&tracker_list_alert::trackers, by_value()))
+        ;
 }
 
 #ifdef _MSC_VER
