@@ -200,6 +200,9 @@ namespace libtorrent {
 		// This flag is here for backwards compatibility.
 		static constexpr create_flags_t canonical_files_no_tail_padding = 9_bit;
 
+		// hidden
+		static constexpr create_flags_t allow_odd_piece_size = 31_bit;
+
 		// The ``piece_size`` is the size of each piece in bytes. It must be a
 		// power of 2 and a minimum of 16 kiB. If a piece size of 0 is
 		// specified, a piece_size will be set automatically.
